unit uArcoConSalidaProgramable;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  Classes,
  uEstados,
  SysUtils, xmatdefs, usimplex, umipsimplex, uglobs,
  uNodos,
  uArcos,
  ucosa, ufechas, uconstantesSimSEE,
  uFichasLPD;

resourcestring
  rsArcoSalidaProgramable = 'Arco con salida programable.';

type

  { TFichaArcoConSalidaProgramable }

  TFichaArcoConSalidaProgramable = class(TFichaArco)
    NPasosDePreAviso: integer; // anticipación a la desconexión
    NPasosDeDesconexion: integer; // tiempo que permanece desconectado
    NPasosAntesNuevaProg: integer; // tiempo luego de volver de una desconexión
    // que debe transcurrir antes de aceptar una nueva programación.
    CostoPorDesconexion: NReal; // [USD] por mandar a desconectar.

    constructor Create(capa: integer; fecha_: TFecha; periodicidad_: TPeriodicidad;
      rendimiento, peaje, PMax: TDAofNReal; NPasosDePreAviso_,
  NPasosDeDesconexion_, NPasosAntesNuevaProg_: integer;
  CostoPorDesconexion_: NREal;
  PagoPorDisponibilidad_USD_MWh: NReal );

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure Free; override;

    Class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
  end;

  { TArcoConSalidaProgramable }

  TArcoConSalidaProgramable = class(TArco)
  public
    X_Desc_Ini: integer; // valor inicial de la variable de estado.

    // Variable de estado
    X_Desc, Xs_Desc: integer;
    dCF: NReal;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades, lpd: TFichasLPD; Entrada, Salida: TNodo; X_Desc_Ini: integer);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure PrepararMemoria(globs: TGlobs); override;
    class function DescClase: string; override;

    procedure prepararPaso_ps; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;
    procedure opt_cargue(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_leerSolucion(s: TSimplex); override;


    procedure optx_nvxs(var ixr, ixd, iauxNReal, iauxInt: integer); override;
    procedure optx_RegistrarVariablesDeEstado(adminEstados: TAdminEstados); override;

    procedure PosicionarseEnEstrellita; override;
    procedure ActualizarEstadoGlobal( flg_Xs: boolean ); override;
    procedure EvolucionarEstado; override;
    procedure Sim_Cronica_Inicio; override;


    function getNombreVar(ivar: integer; var nombre: string): boolean; override;

    procedure dump_Variables(var f: TextFile; charIndentacion: char); override;

    procedure PubliVars; override;
    procedure Free; override;

    // esta función hace el TypeCast por comodidad
    function fpa: TFichaArcoConSalidaProgramable;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
  end;

procedure AlInicio;
procedure AlFinal;

implementation


//-------------------
// Métodos de TArco
//===================

constructor TArcoConSalidaProgramable.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades, lpd: TFichasLPD;
  Entrada, Salida: TNodo; X_Desc_Ini: integer);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, lpd, Entrada, Salida);
  self.X_Desc_Ini := X_Desc_Ini;
end;

constructor TArcoConSalidaProgramable.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('X_Desc_Ini', self.X_Desc_Ini);
  f.EjecutarLectura;
  lpd.Propietario := self;
end;

procedure TArcoConSalidaProgramable.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('X_Desc_Ini', self.X_Desc_Ini);
end;

procedure TArcoConSalidaProgramable.PrepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
end;

class function TArcoConSalidaProgramable.DescClase: string;
begin
  Result := rsArcoSalidaProgramable;
end;



procedure TArcoConSalidaProgramable.prepararPaso_ps;
var
  dx: integer;
begin
  if X_Desc < 0 then
    dx := 1
  else if X_Desc > 1 then
    dx := -1
  else if X_Desc = 1 then
    dx := -(fpa.NPasosDeDesconexion + fpa.NPasosAntesNuevaProg)
  else // X_Desc = 0
    dx := fpa.NPasosDePreAviso;

  {$IFDEF DEV_CF_ESTRELLA}
  if globs.EstadoDeLaSala = CES_OPTIMIZANDO then
    dCF := globs.CF.deltaCosto_vxd_estrella_(ixd, globs.kPaso_ + 1, dx) * globs.fActPaso
  else
  {$ENDIF}
    dCF := globs.CF.deltaCosto_vxd_continuo(ixd, globs.kPaso_ + 1, dx) * globs.fActPaso;

end;


procedure TArcoConSalidaProgramable.opt_nvers(var ivar, ivae, ires: integer);
begin
  if (X_Desc >= -fpa.NPasosAntesNuevaProg) and (X_Desc <> 1) then
  begin
    Self.ivar := ivar;
    ivar := ivar + globs.NPostes;
    if X_Desc = 0 then
    begin
      Inc(ivar);
      self.ivae := ivae;
      Inc(ivae);
    end;
  end;
end;


procedure TArcoConSalidaProgramable.optx_nvxs(
  var ixr, ixd, iauxNReal, iauxInt: integer);
begin
  Self.ixd := ixd;
  ixd := ixd + 1;
end;

procedure TArcoConSalidaProgramable.optx_RegistrarVariablesDeEstado(
  adminEstados: TAdminEstados);
begin
  adminEstados.Registrar_Discreta(
    ixd,
    fpa.NPasosDeDesconexion + fpa.NPasosDePreAviso + fpa.NPasosAntesNuevaProg + 1,
    Self.Nombre + '_XDesc', 'pasos');
end;

procedure TArcoConSalidaProgramable.PosicionarseEnEstrellita;
begin
  X_Desc := globs.CF.xd[ixd] - (fpa.NPasosDeDesconexion + fpa.NPasosAntesNuevaProg);
end;

procedure TArcoConSalidaProgramable.ActualizarEstadoGlobal( flg_Xs: boolean );
begin
  globs.CF.xd[ixd] := X_Desc + fpa.NPasosDeDesconexion + fpa.NPasosAntesNuevaProg;
end;

procedure TArcoConSalidaProgramable.EvolucionarEstado;
begin
  X_Desc := Xs_Desc;
end;

procedure TArcoConSalidaProgramable.Sim_Cronica_Inicio;
begin
  inherited Sim_Cronica_Inicio;
  X_Desc := X_Desc_Ini;
end;


procedure TArcoConSalidaProgramable.opt_cargue(s: TSimplex);
var
  ibaseResSal, ibaseResEnt: integer;
  iposte: integer;
begin
  if (X_Desc >= -fpa.NPasosAntesNuevaProg) and (X_Desc <> 1) then
  begin
{$IFDEF SPXCONLOG}
    for iposte := 0 to globs.NPostes - 1 do
      s.set_NombreVar(ivar + iposte, Nombre + '_P[MW]' + IntToStr(iposte + 1));
{$ENDIF}

    // Tenemos que aportar a las restricciones de
    // demanda de los nodos de
    ibaseResEnt := NodoA.ires;
    ibaseResSal := NodoB.ires;
    for iposte := 0 to globs.NPostes - 1 do
    begin
      s.pon_e(ibaseResEnt + iposte, ivar + iposte, -1);
      s.pon_e(ibaseResSal + iposte, ivar + iposte, pa.rendimiento_[iposte]);
      // Ahora agregamos a la función de UTILIDAD (-costo)
      s.pon_e(s.nf, ivar + iposte, -pa.peaje_[iposte] * globs.durpos[iposte]);
    end;

    if X_Desc = 0 then
      // estoy en condiciones de dar el pre-aviso.
      s.pon_e(s.nf, ivar + globs.NPostes, -(dCF + fpa.CostoPorDesconexion))
    else
      s.acum_e(s.nf, s.nc, -dCF);
  end;
end;

procedure TArcoConSalidaProgramable.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  iposte: integer;
begin
  if (X_Desc >= -fpa.NPasosAntesNuevaProg) and (X_Desc <> 1) then
  begin
    // Le fijamos como cota máxima PMax a la potencia en todos los postes
    for iposte := 0 to globs.NPostes - 1 do
      s.cota_sup_set(ivar + iposte, pa.PMax_[iposte]);

    if X_Desc = 0 then
      TMIPSimplex(s).set_Entera(ivae, ivar + globs.NPostes, 1);
  end;
end;

procedure TArcoConSalidaProgramable.opt_leerSolucion(s: TSimplex);
var
  iposte: integer;
  m: NReal;
  P: NReal;

begin
  CostoDirectoDelPaso := 0;

  vclear(P_Entrante);
  vclear(P_NodoA);
  vclear(P_NodoB);
  vclear(costo_congestion);

  if (X_Desc >= -fpa.NPasosAntesNuevaProg) and (X_Desc <> 1) then
  begin

    // recuperamos los valores de Potencia despachada
    for iposte := 0 to globs.NPostes - 1 do
    begin
      P := s.xval(ivar + iposte);
      P_Entrante[iposte] := P;
      if pa.flg_SumarPeajeAlCDP then
      begin
        m := P * pa.peaje_[iposte] * globs.DurPos[iposte];
        costo[iposte] := m;
        CostoDirectoDelPaso := CostoDirectoDelPaso + m;
      end;
      costo_congestion[iposte] := -s.xmult(ivar + iposte);
      P_NodoA[iposte] := -P;
      P_NodoB[iposte] := P * pa.rendimiento_[iposte];
    end;

    if X_Desc = 0 then
    begin
      if trunc(s.xval(ivar + globs.NPostes) + 0.2) = 1 then
      begin
        Xs_Desc := fpa.NPasosDePreAviso;
        CostoDirectoDelPaso := CostoDirectoDelPaso + fpa.CostoPorDesconexion;
      end
      else
        Xs_Desc := 0; // continuo sin tomar desición
    end
    else
    begin
      if X_Desc < 0 then
        Xs_Desc := X_Desc + 1
      else
        Xs_Desc := X_Desc - 1;
    end;
  end
  else
  begin
    // estoy en la zona de desconexión.
    if X_Desc < 0 then
      Xs_Desc := X_Desc + 1
    else // XDesc = 1
      Xs_Desc := -(fpa.NPasosDeDesconexion + fpa.NPasosAntesNuevaProg);
  end;

end;

function TArcoConSalidaProgramable.getNombreVar(ivar: integer;
  var nombre: string): boolean;
begin
  if (ivar >= self.ivar) and (ivar < self.ivar + globs.NPostes) then
  begin
    nombre := self.Nombre + '_P[MW]' + IntToStr(ivar - self.ivar + 1);
    Result := True;
  end
  else
    Result := False;
end;


procedure TArcoConSalidaProgramable.dump_Variables(var f: TextFile;
  charIndentacion: char);
begin
  inherited dump_Variables(f, charIndentacion);
  writeln(f);
end;

procedure TArcoConSalidaProgramable.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('P', '[MW]', 6, 1, P_Entrante, True, True);
  PublicarVariableNI('X_Desc', 'pasos', X_Desc, True);
  PublicarVariableNR('dCF', 'USD', 12, 2, dCF, True);
end;

procedure TArcoConSalidaProgramable.Free;
begin
  inherited Free;
end;

function TArcoConSalidaProgramable.fpa: TFichaArcoConSalidaProgramable;
begin
  Result := pa as TFichaArcoConSalidaProgramable;
end;

class function TArcoConSalidaProgramable.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list:TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TArcoConSalidaProgramable, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('X_Desc_Ini', TArcoConSalidaProgramable(nil).X_Desc_Ini, TDataTypeIntegerOfCosa, '', 1, -1);
  result:=list;
end;

//---------------------
//Metodos de TFichaArco
//=====================

constructor TFichaArcoConSalidaProgramable.Create(capa: integer; fecha_: TFecha;
  periodicidad_: TPeriodicidad; rendimiento, // rendimientos
  peaje, //[USD/MWh] costo variable por transito
  PMax: TDAofNReal; //[MW] potencia máxima admisible
  //[USD/MWh] costo variable por transito
  NPasosDePreAviso_, NPasosDeDesconexion_, NPasosAntesNuevaProg_: integer;
  CostoPorDesconexion_: NREal;//[MW] potencia máxima admisible
  PagoPorDisponibilidad_USD_MWh: NReal );
begin
  inherited Create(capa, fecha_, periodicidad_,
    Rendimiento,
    peaje,
    PMax,
    1, // fd
    0, // tRepHoras
    True, // ConsidearrPeajeEnElDespacho
    True, // SumarPeajeAlCDP: boolean;
    1.0, //FactorPeajeCDP: NReal);
    PagoPorDisponibilidad_USD_MWh ); // pago por MW-h puesto a disposición
  NPasosDePreAviso := NPasosDePreAviso_;
  NPasosDeDesconexion := NPasosDeDesconexion_;
  NPasosAntesNuevaProg := NPasosAntesNuevaProg_;
  CostoPorDesconexion := CostoPorDesconexion_;
end;

constructor TFichaArcoConSalidaProgramable.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('NPasosDePreAviso', NPasosDePreAviso);
  f.rd('NPasosDeDesconexion', NPasosDeDesconexion);
  f.rd('NPasosAntesNuevaProg', NPasosAntesNuevaProg);
  f.rd('CostoPorDesconexion', CostoPorDesconexion);
  f.EjecutarLectura;
end;

procedure TFichaArcoConSalidaProgramable.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('NPasosDePreAviso', NPasosDePreAviso);
  f.wr('NPasosDeDesconexion', NPasosDeDesconexion);
  f.wr('NPasosAntesNuevaProg', NPasosAntesNuevaProg);
  f.wr('CostoPorDesconexion', CostoPorDesconexion);
end;


procedure TFichaArcoConSalidaProgramable.Free;
begin
  inherited Free;
end;

class function TFichaArcoConSalidaProgramable.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list:TDataColumnListOfCosa;
begin
  //list:=TDataColumnListOfCosa.Create(TFichaArcoConSalidaProgramable, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('NPasosDePreAviso', TFichaArcoConSalidaProgramable(nil).NPasosDePreAviso, TDataTypeIntegerOfCosa, '', 1, -1);
  list.AddDataColumn('NPasosDeDesconexion', TFichaArcoConSalidaProgramable(nil).NPasosDeDesconexion, TDataTypeIntegerOfCosa, '', 1, -1);
  list.AddDataColumn('NPasosAntesNuevaProg', TFichaArcoConSalidaProgramable(nil).NPasosAntesNuevaProg, TDataTypeIntegerOfCosa, '', 1, -1);
  list.AddDataColumn('CostoPorDesconexion', TFichaArcoConSalidaProgramable(nil).CostoPorDesconexion, TDataTypeDoubleOfCosa, '', 1, -1);
  result:=list;
end;
procedure AlInicio;
begin
  registrarClaseDeCosa(TArcoConSalidaProgramable.ClassName, TArcoConSalidaProgramable);
  registrarClaseDeCosa(TFichaArcoConSalidaProgramable.ClassName,
    TFichaArcoConSalidaProgramable);
end;

procedure AlFinal;
begin
end;

end.



